'use strict';
angular.module('aom.services')
  .factory('aomMediaSrv', function ($timeout, $state, $cordovaCamera, $cordovaCapture, $q, $ionicActionSheet, $cordovaFile, aomComponentsSrv, aomResourcesSrv, aomFileSystemSrv, aomMediaPluginSrv, aomPopupSrv, aomLogSrv, constants, aomShareSrv) {

    var mediaSrv = {};

    mediaSrv.data = {
      isSelectingMedia: false
    };

    function createOptions(source, media) {
      return {
        destinationType: ionic.Platform.isEdge() ? Camera.DestinationType.NATIVE_URI : Camera.DestinationType.FILE_URI,
        sourceType: source,
        mediaType: media,
        allowEdit: false,
        encodingType: Camera.EncodingType.JPEG,
        saveToPhotoAlbum: false,
        correctOrientation: true
      };
    }

    function optionsForType(object) {
      var source;
      var media;
      var options;
      switch (object.key) {
        case 'Photo':
          source = Camera.PictureSourceType.CAMERA;
          media = Camera.MediaType.PICTURE;
          options = createOptions(source, media);
          break;
        case 'Video':
          options = { limit: 1, duration: 30 };
          break;
        case 'Gallery':
          source = Camera.PictureSourceType.PHOTOLIBRARY;
          if (object.parameters.source.photo && object.parameters.source.video) {
            media = Camera.MediaType.ALLMEDIA;
          } else {
            media = object.parameters.source.photo ? Camera.MediaType.PICTURE : Camera.MediaType.VIDEO;
          }
          options = createOptions(source, media);
          break;
      }
      return options;
    }

    function resolveFileURL(media) {
      var response = 'No Path found';
      if (media === null) {
        return $q.reject(response);
      } else if (angular.isArray(media)) {
        response = (media[0].localURL);
      } else if (ionic.Platform.isAndroid() && media.substring(0, 4) !== 'file') {
        response = 'file://' + media;
      } else {
        response = media;
      }
      return $q.when(response);
    }

    function copyFileToDisk(imageUrl) {
      imageUrl = imageUrl.split('?')[0];
      var name = imageUrl.substr(imageUrl.lastIndexOf('/') + 1);
      var location = imageUrl.substr(0, imageUrl.lastIndexOf('/') + 1);
      return aomFileSystemSrv.copyToLocalData(location, name, aomComponentsSrv.generateUniqueId() + name.replace(constants.common.fileNameSeperator, ''));
    }

    function saveMedia(object) {
      var options = optionsForType(object);
      var operation = object.key === 'Video' ? $cordovaCapture.captureVideo : $cordovaCamera.getPicture;
      return operation(options)
        .then(function (media) {
          return resolveFileURL(media);
        })
        .then(function (nativePath) {
          return copyFileToDisk(nativePath);
        });
    }

    function checkVideoLength(fileEntry, type) {
      var deferred = $q.defer();
      var path = ionic.Platform.isEdge() ? fileEntry.fullPath : fileEntry.nativeURL;
      aomMediaPluginSrv.getVideoInfo(path)
        .then(function (response) {
          if (response.duration < 31) {
            deferred.resolve({ name: fileEntry.name, type: type });
          } else {
            aomFileSystemSrv.removeAttachment(fileEntry.name);

            aomPopupSrv.alert('videoSavingFailed', {
              title: aomResourcesSrv.data.resources.Common.Error,
              template: aomResourcesSrv.data.resources.Media.LongVideoDuration,
              onDismiss: function () { }
            });
            const err = { message: aomResourcesSrv.data.resources.Media.LongVideoDuration, error: 'Video taken was ' + response.duration + ' seconds and thus was not saved' };
            aomLogSrv.log(err, aomLogSrv.levels.user, aomLogSrv.severities.err);
            deferred.reject(err);
          }
        })
        .catch(function (err) {
          if (!hasUserCancelled(err)) {
            aomPopupSrv.alert('videoSavingFailed', {
              title: aomResourcesSrv.data.resources.Common.Error,
              template: aomResourcesSrv.data.resources.Media.FailedToSaveVideo,
              onDismiss: function () { }
            });
            aomLogSrv.log({ message: aomResourcesSrv.data.resources.Media.FailedToSaveVideo, error: err }, aomLogSrv.levels.user, aomLogSrv.severities.err);
            deferred.reject(err);
          }
        });
      return deferred.promise;
    }

    function checkMediaAndGetMetaData(fileEntry) {
      var defered = $q.defer();
      fileEntry.file(function (metadata) {
        var type = ionic.Platform.isAndroid() ? metadata.type : mediaSrv.getMediaType('.' + fileEntry.name.split('.').pop());
        if (mediaSrv.checkMediaType('video', type)) {
          checkVideoLength(fileEntry, type)
            .then(function (fileObject) {
              defered.resolve(fileObject);
            })
            .catch(function (err) {
              defered.reject(err);
            });
        } else {
          defered.resolve({ name: fileEntry.name, type: type });
        }
      });
      return defered.promise;
    }

    function getMetaData(fileEntry) {
      var defered = $q.defer();
      fileEntry.file(function (metadata) {
        var type = ionic.Platform.isAndroid() ? metadata.type : mediaSrv.getMediaType('.' + fileEntry.name.split('.').pop());
        defered.resolve({ name: fileEntry.name, type: type, size: metadata.size });
      });
      return defered.promise;
    }

    function checkErrorString(err) {
      return err === 'no image selected' || err === 'User didn\'t choose a file.' || err === 'User didn\'t capture a photo.' ||
        err === 'Camera cancelled.' || err === 'Selection cancelled.' || err === 'has no access to assets';
    }

    function hasUserCancelled(err) {
      return err && ((err.hasOwnProperty('code') && err.code === 3) || checkErrorString(err));
    }

    mediaSrv.showMediaOptions = function (parameters, $scope) {
      var defered = $q.defer();
      var attachmentTypes = [];

      if (parameters === undefined) {
        parameters = { source: { photo: true, video: true, gallery: true } };
      }

      if (parameters.source.photo) {
        attachmentTypes.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.PhotoAdd, key: 'Photo', parameters: parameters });
      }
      if (parameters.source.video) {
        attachmentTypes.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.VideoAdd, key: 'Video', parameters: parameters });
      }
      if (parameters.source.gallery) {
        attachmentTypes.push({ text: aomResourcesSrv.data.resources.AomActionSheetOptions.GalleryAdd, key: 'Gallery', parameters: parameters });
      }

      if (aomResourcesSrv.data.resources.Features.IsEnterprise) {
        if (parameters.source.sharedFiles) {
          attachmentTypes.push({ text: aomResourcesSrv.data.resources.AnyFile.AddDocument, key: 'Files' });
        }
      }

      $ionicActionSheet.show({
        buttons: attachmentTypes,
        titleText: aomResourcesSrv.data.resources.AomActionSheetOptions.Title,
        cancelText: aomResourcesSrv.data.resources.Common.Cancel,
        cancel: function () {
          return true;
        },
        buttonClicked: function (index, object) {
          mediaSrv.data.isSelectingMedia = true;
          if (object.key === 'Files') {
            aomShareSrv.show($scope)
              .then(function (path) {
                return copyFileToDisk(path);
              })
              .then(function (fileEntry) {
                return getMetaData(fileEntry);
              })
              .then(function (fileObject) {
                if (fileObject.size / 1024.0 / 1024.0 > aomResourcesSrv.data.resources.AnyFile.FileSizeLimitInMB) {
                  return $q.reject({ name: 'ExceedsMaxSizeError' });
                } else {
                  defered.resolve({ file: fileObject, source: object.key });
                }
              })
              .catch(function (err) {
                aomLogSrv.log({ message: aomResourcesSrv.data.resources.Media.FailedToSaveMedia, error: err }, aomLogSrv.levels.user, aomLogSrv.severities.err);
                if (err && err.name) {
                  if (err.name !== 'NotSelected' && aomResourcesSrv.data.resources.AnyFile[err.name])
                    aomPopupSrv.alert('AnyFile' + err.name, {
                      title: aomResourcesSrv.data.resources.Common.Error,
                      template: aomResourcesSrv.data.resources.AnyFile[err.name],
                      onDismiss: function () { }
                    });
                } else {
                  defered.reject(err);
                }
              });
          } else {
            saveMedia(object)
              .then(function (fileEntry) {
                return checkMediaAndGetMetaData(fileEntry);
              })
              .then(function (fileObject) {
                defered.resolve({ file: fileObject, source: object.key });
              })
              .catch(function (err) {
                if (!hasUserCancelled(err)) {
                  defered.reject(err);
                } else {
                  defered.reject('UserCancelled');
                }
              });
          }
          return true;
        }
      });
      return defered.promise;
    };

    mediaSrv.getMediaType = function (type) {
      return constants.extensionsToMimeTypes[type.toLowerCase()];
    };

    mediaSrv.checkMediaType = function (mediaType, mimeType) {
      return (constants.mimeTypes[mediaType].indexOf(mimeType) !== -1);
    };

    mediaSrv.getFileType = function (mimeType) {
      for (var key in constants.mimeTypes) {
        if (constants.mimeTypes.hasOwnProperty(key)) {
          if (constants.mimeTypes[key].indexOf(mimeType) !== -1) {
            return key;
          }
        }
      }
    };

    return mediaSrv;
  });